import click


from indico_install.utils import run_cmd


@click.command("update")
def update():
    """
    Generates image updates and scaling updates from a YAML file
    """
    # Read the cluster dump to grab the deployment names for updates
    # Read the update YAML to match the deployment names
    # Generate the script
    # Upload the script to google storage with proper permissions
    output = run_cmd(
        "kubectl get deploy -o wide  | grep indico | awk '{print $1\"=\"$7}'",
        silent=True,
    )

    images = {
        x[0]: x[1] for x in [y.split("=") for y in output.split("\n") if y.strip()]
    }

    for deploy, image in images.items():
        click.echo(f"kubectl set image deployment/{deploy} {deploy}={image}")

    output = run_cmd(
        "kubectl get statefulset -o wide  | grep indico | awk '{print $1\"=\"$5}'",
        silent=True,
    )

    images = {
        x[0]: x[1] for x in [y.split("=") for y in output.split("\n") if y.strip()]
    }

    for statefulset, image in images.items():
        click.echo(f"kubectl set image statefulset/{statefulset} {statefulset}={image}")

    click.echo(run_cmd("khash", silent=True))



import re
from collections import defaultdict

import click

from indico_install.utils import run_cmd

status_order = defaultdict(lambda: len(status_order))
status_order.update(
    {
        "STATUS": 0,
        "Running": 1,
        "Terminating": 2,
        "ContainerCreating": 2,
        "CrashLoopBackOff": 3,
        "Unknown": 10,
    }
)


@click.group("pod")
@click.pass_context
def pod(ctx):
    """Utility commands to interact with pods"""


@pod.command("exec")
@click.pass_context
@click.argument("podname", required=True)
@click.option(
    "-c", "--command", default="bash", help="Command to run", show_default=True
)
def exc(ctx, podname, command="bash"):
    """
    Execute a command in a pod interactively through a TTY.

    Args:
        <PODNAME> pod name substring to exec into
    """
    pod = run_cmd(
        f"kubectl get pods --no-headers | grep {podname} | head -n 1 | awk '{{print $1}}'",
        silent=True,
    )
    run_cmd(["kubectl", "exec", "-it", pod, command], tty=True)


@pod.command("ls")
@click.pass_context
@click.argument("podname", required=False)
def list_pods(ctx, podname):
    """
    List pods with optional <PODNAME> filter.
    """
    output = run_cmd("kubectl get pods -o wide", silent=True)
    if not output or "No resources" in output:
        click.secho(output, fg="yellow")
        return
    podname = podname or ""

    lines = output.split("\n")
    header = lines[0]
    output_lines = [line for line in lines[1:] if line.strip() and podname in line]
    sort_lines = sorted(
        enumerate([re.split(r"\s{2,}", line) for line in output_lines]),
        key=lambda parts: (
            status_order[parts[1][2]],
            -int(parts[1][1].split("/")[0]),
            parts[1][0],
        ),
    )

    click.echo(header)
    click.echo("\n".join(output_lines[idx] for idx, _ in sort_lines))



import click
import pkg_resources

from indico_install.cluster_manager import ClusterManager
from indico_install.config import yaml


def init(location):
    def wrapper():
        cluster_manager = ClusterManager()
        if cluster_manager.cm_exists:
            click.secho(
                f"Cluster config already exists. "
                "Please back this config up before running init.",
                fg="red",
            )
            return

        template = pkg_resources.resource_string(location, "cluster.yaml")
        cluster_manager.edit_cluster_config(changes=yaml.safe_load(template))
        cluster_manager.save()

        click.secho(f"Created Cluster Config", fg="green")

    return wrapper



import os
import tarfile
from pathlib import Path

import click
from google.cloud import storage

from indico_install.utils import options_wrapper, run_cmd, current_user, string_to_tag

UPLOAD_BUCKET = "indico-templates"


@click.command("upload")
@click.pass_context
@click.option(
    "-t",
    "--tag",
    required=True,
    multiple=True,
    help="Subdirectory(s) of {UPLOAD_BUCKET} to upload to",
)
@options_wrapper(check_services=True)
def upload(ctx, tag, *, deployment_root, services_yaml, **kwargs):
    """
    Upload the gzipped templates directory and services.yaml to indico-templates/<TAG> for each tag.
    For each <RENDER>, indico render for those values and publish the generated templates also to indico-templates/<TAG>.
    Will only overwrite

    Requires service account key to be base64-encoded and stored in the GOOGLE_SERVICE_ACCOUNT environment variable for authentication
    """
    deployment_root = Path(deployment_root)
    templates_dir = deployment_root / "templates"

    upload_user = current_user()
    assert upload_user, "Current user is not correctly authenticated with gcloud"

    if upload_user == "service_acct":
        try:
            run_cmd(
                "echo $GOOGLE_SERVICE_ACCOUNT | base64 -d > google.json", silent=True
            )
            os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = str(Path("google.json"))
            client = storage.Client()
            upload_bucket = client.get_bucket(UPLOAD_BUCKET)
        except Exception as e:
            click.secho(f"Unable to access bucket for uploads: {e}", fg="red")
            return

    if not templates_dir.is_dir() or not services_yaml.is_file():
        click.secho(f"Could not find {templates_dir} or {services_yaml}", fg="red")
        return

    with tarfile.open(deployment_root / "templates.tar.gz", "w:gz") as tar:
        click.secho(f"Tar-ing templates {templates_dir}")
        tar.add(templates_dir, arcname="")

    final_tags = [string_to_tag(t) for t in tag]

    for upload_file in [services_yaml, deployment_root / "templates.tar.gz"]:
        for upload_path in final_tags:
            if upload_user == "service_acct":
                upload_bucket.blob(
                    f"{upload_path}/{upload_file.name}"
                ).upload_from_filename(filename=str(upload_file))
            else:
                run_cmd(f"gsutil cp {upload_file} gs://{UPLOAD_BUCKET}/{upload_path}/")

    click.secho("Uploads complete! Tags:\n - " + "\n - ".join(final_tags), fg="green")
    return final_tags



import json
import re

import click
from click import confirm, secho

from indico_install.config import ConfigsHolder
from indico_install.utils import options_wrapper, get_non_matching_images, get_nginx_conf


@click.group("pull")
@click.pass_context
def pull(ctx):
    """Pull the latest information from the cluster into services yaml file"""
    pass


@pull.command("all")
@click.pass_context
@options_wrapper(check_services=True)
def pull_all(ctx, **kwargs):
    """Pull both backend images and frontend hashes to your services yaml"""
    ctx.invoke(pull_images, **kwargs)
    ctx.invoke(pull_frontend_hashes, **kwargs)


@pull.command("images")
@click.pass_context
@options_wrapper(check_services=True)
def pull_images(ctx, *, services_yaml, yes, **kwargs):
    """Pull images from cluster"""
    configs = ConfigsHolder(config=services_yaml)

    for app, _, _, saved_image, cluster_image in get_non_matching_images(
        configs, only_first=True
    ):
        secho(f"{app}:\nOn Cluster: {cluster_image}\nOn Disk: {saved_image}")
        if yes or confirm("Save these changes to disk?"):
            secho(f"Saving changes for {app}", fg="green")
            configs["images"][app] = cluster_image
            configs.save()
        else:
            secho(f"Skipping changes for {app}", fg="yellow")


@pull.command("clients")
@click.pass_context
@options_wrapper(check_services=True)
def pull_frontend_hashes(ctx, *, services_yaml, yes, **kwargs):
    """Pull frontend hashes from cluster"""
    configs = ConfigsHolder(config=services_yaml)
    output = get_nginx_conf()
    output_string = json.loads(output)

    for app, _hash in configs["frontend"]["hash"].items():
        try:
            cluster_hash = re.search(fr"/{app}/(\w+);", output_string).group(1)
        except Exception:
            secho(f"Could not find {app} in cluster app-edge nginx.conf", fg="red")
        else:
            if cluster_hash != _hash.strip():
                secho(
                    f"{app} frontend hash\nOn Cluster: {cluster_hash}\nOn Disk: {_hash}"
                )
                if yes or confirm("Save these changes to disk?"):
                    secho(f"Saving changes for frontend {app}", fg="green")
                    configs["frontend"]["hash"][app] = cluster_hash
                    configs.save()
                else:
                    secho(f"Skipping changes for {app}", fg="yellow")

    secho("Done syncing changes", fg="green")


@pull.command("client")
@click.pass_context
@options_wrapper(check_services=True)
def pull_frontend_hash(ctx, *, services_yaml, yes, **kwargs):
    """Pull frontend hash from cluster"""
    configs = ConfigsHolder(config=services_yaml)
    output = get_nginx_conf()
    output_string = json.loads(output)
    app_hash = configs["frontend"]["hash"]

    try:
        cluster_hash = (
            re.search(rf"set \$clientversion [^;]*", output_string)
            .group(0)
            .split(" ")[-1]
        )
    except Exception:
        secho(f"Could not find client version in cluster app-edge nginx.conf", fg="red")
    else:
        if cluster_hash != app_hash.strip():
            secho(f"Frontend hash\nOn Cluster: {cluster_hash}\nOn Disk: {app_hash}")
            if yes or confirm("Save these changes to disk?"):
                secho(f"Saving changes for frontend", fg="green")
                configs["frontend"]["hash"] = cluster_hash
                configs.save()
            else:
                secho(f"Skipping frontend changes", fg="yellow")

    secho("Done syncing changes", fg="green")



import click

from indico_install.generate.cluster_info import cluster_info
from indico_install.generate.update import update



@click.group("generate")
def generate():
    """Scale and configure cluster from dump"""
    pass


generate.add_command(update)
generate.add_command(cluster_info)



from click import prompt
from indico_install.utils import run_cmd
from indico_install.infra.input_utils import postgres_input, auth_with_gsutil

def aks_setup(deployment_root):
    """
    Make sure we have the necessary scripts and auth for installation
    """
    assert run_cmd("which az", silent=True), "Please install the Azure CLI"
    assert run_cmd("which gcloud", silent=True) and run_cmd("which gsutil", silent=True), "Please install the gcloud and gsutil CLI"
    assert 'name' in run_cmd("az account show | grep name", silent=True), "Please log in using az login"
    authed = auth_with_gsutil(deployment_root, raise_missing_key=True)
    assert authed, "Unable to auth with gcloud service account"


def ask_for_infra_input(conf):
    cluster = conf.get("cluster", {})
    conf["cluster"] = cluster
    storage_acct_name = prompt("What is the name of the storage account", type=str, default=cluster.get("storage_acct"))
    storage_acct_key = prompt("What is a storage account key", type=str, default=cluster.get("storage_acct_key"))
    rwx_fs = prompt("Read-write fileshare name", type=str, default=conf["clusterVolumes"]["rwx"]["azureFile"]["shareName"])
    rox_fs = prompt("Indicoapi data fileshare name", type=str, default=conf["clusterVolumes"]["rox"]["azureFile"]["shareName"])

    conf["cluster"].update({
        "storage_acct": storage_acct_name,
        "storage_acct_key": storage_acct_key,
    })
    conf["clusterVolumes"]["rwx"]["azureFile"]["shareName"] = rwx_fs
    conf["clusterVolumes"]["rox"]["azureFile"]["shareName"] = rox_fs

    postgres_input(conf)

    conf.save()



import click
from indico_install.infra.eks.config import session
from indico_install.utils import decodeb64, run_cmd


@session
def validate(config, session=None):
    """
    Validates the following:
    - RDS database for postgres has a certain instance size
    - db is configured with parallel workers
    - does not validate Redis (we assume in-cluster here)
    """
    info = config["postgres"]["app"]
    assert info.get("host"), "Database info not provided"
    try:
        db = session.RDSClient.describe_db_instances(
            Filters=[
                {
                    "Name": "db-cluster-id"
                    if "cluster" in info["host"]
                    else "db-instance-id",
                    "Values": [info["host"].split(".")[0]],
                }
            ]
        )["DBInstances"][0]
    except Exception:
        raise Exception("Unable to locate database")

    assert "postgres" in db["Engine"].lower(), "DB not a postgres instance!"
    assert db["DBInstanceStatus"] == "available"
    if "aurora" not in db["Engine"].lower():
        # Aurora DBs are dynamically provisioned
        assert db["AllocatedStorage"] >= 500, "DB must have at least 500GB of storage"
    assert (
        "xlarge" in db["DBInstanceClass"]
    ), "Database instance must be at least an xlarge"
    param_groups = [
        p["DBParameterGroupName"]
        for p in db["DBParameterGroups"]
        if p["ParameterApplyStatus"] == "in-sync"
    ]

    all_params = {}
    params_we_want = {
        "max_worker_processes": {"fn": lambda x: x and int(x) >= 80, "doc": ">= 80"},
        "force_parallel_mode": {"fn": lambda x: x and int(x) == 1, "doc": "== 1"},
        "max_parallel_workers_per_gather": {
            "fn": lambda x: x and int(x) >= 16,
            "doc": ">= 16",
        },
    }

    paginator = session.RDSClient.get_paginator("describe_db_parameters")
    for pg in param_groups:
        param_pages = paginator.paginate(DBParameterGroupName=pg)
        for page in param_pages:
            all_params.update(
                {
                    p["ParameterName"]: p.get("ParameterValue")
                    for p in page["Parameters"]
                    if p["ParameterName"] in params_we_want
                }
            )
    bad_params = [k for k, v in all_params.items() if not params_we_want[k]["fn"](v)]
    assert not bad_params, ", ".join(
        [f"{k} is not {params_we_want[k]['doc']}" for k in bad_params]
    )
    check_database(info)


def check_database(info):
    # ask for user, password, db_name to connect to
    host = info["host"]
    user = info["user"]
    password = info["password"]
    db_name = "indico"
    if not run_cmd("command -v pg_isready 2>&1", silent=True).strip():
        click.secho("pg_isready not found on machine - cannot validate db connection.", fg="yellow")
        return
    pgready_command = f"pg_isready -h {host} -U {user}"
    output = run_cmd(pgready_command)
    if "accepting connections" not in output:
        click.secho(f"Unable to connect to postgres {host} from this machine")
        return

    if not run_cmd("command -v psql 2>&1", silent=True).strip():
        click.secho("psql not found on machine - cannot validate db connection.", fg="yellow")
        return

    psql_command = f"PGPASSWORD={decodeb64(password)} psql -h {host} -p 5432 -U {user} {db_name} --list --tuples-only"
    output = run_cmd(psql_command, silent=True)
    tables = {}
    for line in output.splitlines():
        db, owner = [val.strip() for val in line.split(" | ")[:2]]
        tables.update({db: owner} if db else {})

    all_dbs = ["crowdlabel", "cyclone", "elmosfire", "moonbow", "noct"]

    not_found = [db for db in all_dbs if db not in tables]
    assert (
        not not_found
    ), f"The following databases do not exist in your database: {not_found}"



import json
import os
import tempfile
from pathlib import Path
from subprocess import call

import click

from indico_install.cluster_manager import ClusterManager
from indico_install.config import merge_dicts, yaml
from indico_install.helm.apply import apply
from indico_install.helm.render import render
from indico_install.utils import options_wrapper, pretty_diff, string_to_tag

EDITOR = os.environ.get("EDITOR", "vim")


def refresh_cluster_manager(ctx, cluster_manager, deployment_root, yes, backup=False):
    """
    Given recent changes, apply them to the cluster
    """
    cluster_manager.lock()
    try:
        ctx.invoke(
            render,
            cluster_manager=cluster_manager,
            deployment_root=deployment_root,
            allow_image_overrides=backup,
        )
        ctx.invoke(
            apply,
            cluster_manager=cluster_manager,
            deployment_root=deployment_root,
            yes=yes,
        )
    finally:
        cluster_manager.save()
        cluster_manager.unlock()


@click.group("updraft")
def updraft():
    """Manage deployments and versioning"""
    pass


@updraft.command("version")
@click.pass_context
def show_version(ctx):
    """
    Display version details about <TAG>. Defaults to current cluster rendered_tag
    """
    click.secho(f"Known version: {ClusterManager().indico_version}")


@updraft.command("current")
@click.pass_context
def show_current(ctx):
    """
    Display current configmap state (alias for viewing the configmap)
    """
    click.secho(ClusterManager().to_str())


@updraft.command("edit")
@click.pass_context
@click.option(
    "-I", "--interactive", is_flag=True, help=f"Open configmap in your editor",
)
@click.option("-v", "--version", help=f"Update to new updraft version")
@click.option("--force", help=f"")
@click.option("--patch-file", type=click.File("r"), help="JSON-formatted patch file")
@click.argument("patch", required=False)
@options_wrapper()
def edit_configmap(
    ctx, *, interactive, patch, patch_file, version, yes, deployment_root, **kwargs
):
    """
    Edit configmap with some patch, or interactively.
    Only allows editing of the main cluster config portion

    PATCH - if not interactive, and no patch-file is provided,
    pass in a json string to patch the configmap with

    ex: indico updraft edit '{"apiDomain": "api-foo.indico.io"}'
    """
    cluster_manager = ClusterManager()
    if not cluster_manager.cm_exists:
        click.secho(
            "Cannot edit cluster config. Please initialize with indico updraft init",
            fg="red",
        )
        return
    changes = None
    service_state = cluster_manager.clean_services()
    if interactive:
        with tempfile.NamedTemporaryFile(suffix=".tmp") as tf:
            tf.write(
                yaml.dump(
                    cluster_manager.cluster_config, default_flow_style=False
                ).encode("utf-8")
            )
            tf.flush()
            call([EDITOR, tf.name])
            tf.seek(0)
            changes = yaml.safe_load(tf)
    elif patch_file:
        changes = json.load(patch_file)
    elif patch:
        changes = json.loads(patch)

    if not changes and (
        not version or string_to_tag(version) == cluster_manager.indico_version
    ):
        click.secho(f"No changes or new version provided", fg="yellow")
        return

    cluster_manager.edit_cluster_config(
        changes=merge_dicts(changes, service_state), version=string_to_tag(version)
    )

    if yes or click.confirm("Render and apply with changes?"):
        cluster_manager.save(backup=True)
        refresh_cluster_manager(ctx, cluster_manager, deployment_root, yes)


@updraft.command("restore")
@click.pass_context
@options_wrapper()
def restore_version(ctx, deployment_root, yes):
    """
    (Alpha) If an old version of the cluster_manager exists, pull it
    load the original state, and apply
    """
    cluster_manager = ClusterManager()
    if not cluster_manager.load_from_cluster(backup=True):
        click.secho("No backup available!", fg="red")
        return

    cluster_manager.lock()
    try:
        if yes or click.confirm("Apply backup?"):
            refresh_cluster_manager(
                ctx, cluster_manager, deployment_root, yes, backup=True
            )
    finally:
        cluster_manager.unlock()


@updraft.command("init")
@click.pass_context
@options_wrapper()
def init_tracking(ctx, input_yaml=None, yes=False, **kwargs):
    """
    Add or update version tracking to the cluster (idempotent)
    """
    if input_yaml and not Path(input_yaml).is_file():
        click.secho(
            f"Provided input yaml {input_yaml} does not exist. Ignoring", fg="yellow"
        )
        input_yaml = None
    cluster_manager = ClusterManager(reconcile=True, input_yaml=input_yaml)
    click.secho(cluster_manager.to_str())
    if yes or click.prompt("Save updated version cluster_manager [yN]"):
        cluster_manager.save()


@updraft.command("diff")
@click.pass_context
@click.argument("versions", required=False, nargs=-1)
@click.option(
    "--all/--no-all",
    "show_all",
    default=False,
    show_default=True,
    help="Include matches in diff",
)
def compare_versions(ctx, versions=None, show_all=False):
    """
    Compare versions
    If no versions are provided, diff rendered_release with current state
    If 1 version, diff current state with version
    If 2 versions, diff the versions
    """
    if len(versions) > 2:
        click.secho(
            "More that 2 versions provided for comparison. Ignoring extra versions"
        )
        versions = versions[:2]
    if versions:
        versions = [string_to_tag(v) for v in versions]

    cluster_manager = ClusterManager()
    if len(versions) == 2:
        diff = cluster_manager.diff(*versions)
    else:
        diff = cluster_manager.diff_version(tag=versions[0] if versions else None)

    pretty_diff(diff, show_all=show_all)



import os
import click
from indico_install.config import merge_dicts
from indico_install.utils import run_cmd
from indico_install.infra.init import init

DEFAULT_NODE_CONFIGS = {
    "image-type": "UBUNTU",
    "disk-type": "pd-standard",
    "scopes": '"https://www.googleapis.com/auth/devstorage.read_only","https://www.googleapis.com/auth/logging.write","https://www.googleapis.com/auth/monitoring","https://www.googleapis.com/auth/servicecontrol","https://www.googleapis.com/auth/service.management.readonly","https://www.googleapis.com/auth/trace.append"',
    "no-enable-autorepair": None,
    "min-nodes": 1,
    "max-nodes": 3,
}

NODE_POOL_TYPES = {
    "default-gpu-pool": {
        "machine-type": "n1-standard-8",
        "accelerator": "type=nvidia-tesla-p4,count=1",
    },
    # N2s do not support GPU and are not supported in us-east4-c
    "default-cpu-pool": {"machine-type": "n1-standard-2"},
    "default-mem-pool": {"machine-type": "n1-standard-8"},
    "default-pool": {"machine-type": "n1-standard-16"},
    "finetune": {
        "machine-type": "n1-standard-8",
        "accelerator": "type=nvidia-tesla-p4,count=1",
    },
}


def nodegroup_args(ng_type, **kwargs):
    args = merge_dicts(DEFAULT_NODE_CONFIGS, NODE_POOL_TYPES[ng_type])
    args.update(kwargs)
    if "accelerator" not in args:
        args["enable-autoscaling"] = None
    return " ".join([f"--{k}" if v is None else f"--{k}={v}" for k, v in args.items()])


@click.group("gke")
@click.pass_context
def gke(ctx):
    """
    Managing a kubernetes cluster on Google Kubernetes Engine
    """
    pass


gke.command("init")(init(__name__))
environment_abbrev = {"production": "prod", "development": "dev"}


@gke.command("create-nodepools")
@click.pass_context
@click.option(
    "--cluster-name", required=True, help="Name of cluster to attach nodepools to"
)
@click.option(
    "--node-pool",
    help=f"additional pools and counts. EX: --node-pool gpu=3 --node-pool finetune=1. Types are {list(NODE_POOL_TYPES.keys())}",
    multiple=True,
)
def create_nodepools(ctx, cluster_name, node_pool=None):
    """
    Creates additional nodepools for existing cluster
    """
    node_pools = {}
    for np in node_pool or []:
        np_type, np_size = np.split("=", 1)
        assert (
            np_type in NODE_POOL_TYPES
        ), f"node_pool type {np_type} is not valid. Please select from {NODE_POOL_TYPES}"
        assert (
            int(np_size) > 0
        ), f"node_pool size {np_size} is not valid. Please specify an int > 0"
        node_pools[np_type] = np_size

    if not node_pools:
        return

    for np, count in node_pools.items():
        run_cmd(
            f"gcloud container node-pools create {np} "
            f"--num-nodes={count} --cluster={cluster_name} " + nodegroup_args(np)
        )


@gke.command("create")
@click.pass_context
@click.argument("environment")
@click.argument("name")
@click.argument("size", type=int)
@click.option("--subnetwork", help="Network to be used")
@click.option(
    "--version", help="GKE Cluster Version", default="1.14.8-gke.33", show_default=True
)
@click.option(
    "--project", help="GKE Project Name", default="new-indico", show_default=True
)
@click.option("--zone", help="GKE zone", default="us-east4-c", show_default=True)
@click.option(
    "-d",
    "--deployment-root",
    default=os.getcwd(),
    show_default=True,
    help="Root directory for installation files",
)
@click.option(
    "--node-pool",
    help=f"additional pools. EX: --node-pool gpu=3 --node-pool finetune=1. Types are {list(NODE_POOL_TYPES.keys())}",
    multiple=True,
)
def create_cluster(
    ctx,
    environment,
    name,
    size,
    subnetwork=None,
    version=None,
    project=None,
    zone=None,
    deployment_root=None,
    node_pool=None,
):
    """
    Creates a GKE cluster through gcloud.

    Don't change the kubernetes version unless absolutely necessary for
        security invulnerabilities/necessary feature

    The network used is default to indico-{environment},
        this will need to be created ahead of time with custom subnets (not auto)

    The subnetwork specified needs to be created in the network
        created in the zone desired.

    Use specific node pools by specifying the nodeSelector in the service
        using indico_pool: <type>. The default pool is default.

    Prod scaling is indico_pool: default = 2, gpu = 1, cpu = 1
    """
    environment = next(
        iter(env for env in {"production", "development"} if environment in env)
    )

    name = name.lower()
    assert environment_abbrev[environment] in name

    size = int(size)
    assert size > 0

    subnetwork = subnetwork or f"{environment}-default"
    network = f"indico-{environment}"

    click.secho(
        f"Creating cluster {name} in {environment} with {size} nodes...",
        fg="red" if environment == "production" else "green",
    )
    # TODO: Check if we need the python2.7 aliasing.
    # If we do, figure out how to install gcloud for python3

    # TODO: this will probably require components installation and will cause the
    # command to fail since its non-interactive.
    run_cmd(
        f"""
        gcloud beta container clusters create {name} \
            --project "{project}" \
            --zone {zone} \
            --username "admin" \
            --cluster-version "{version}" \
            --num-nodes "{size}" \
            --max-nodes "{size + 1}" \
            --min-nodes "{max(1, size - 1)}" \
            --enable-cloud-logging \
            --enable-cloud-monitoring \
            --addons HorizontalPodAutoscaling,HttpLoadBalancing \
            --no-enable-autoupgrade \
            --enable-ip-alias \
            --network "projects/new-indico/global/networks/{network}" \
            --subnetwork "{subnetwork}" \
        """
        + nodegroup_args("default-pool")
    )

    # TODO: Refactor rbac & switch
    # TODO: ensure this actually persists in current session
    run_cmd(
        f"""
        gcloud config set compute/zone {zone}
        gcloud container clusters get-credentials {name} \
            --zone {zone} \
            --project {project}
        """
    )

    email = run_cmd(
        f"""
        cat {deployment_root}/.config/gcloud/configurations/config_default \
            | grep account \
            | awk -F "=" '{{print $2}}'
        """
    )

    user = email.split("@")[0]

    run_cmd(
        f"""kubectl create clusterrolebinding cluster-admin-binding-{user} \
            --clusterrole cluster-admin \
            --user {email}"""
    )
    if node_pool:
        ctx.invoke(create_nodepools, cluster_name=name, node_pool=node_pool)

    # TODO: Wait for nodes to be ready - then run Nvidia-scripts
    # TODO: Nvidia Script



import shutil
import tarfile
import time
from pathlib import Path

import click
from click import secho

from indico_install.config import CLUSTER_NAME
from indico_install.infra.input_utils import auth_with_gsutil
from indico_install.utils import current_user, options_wrapper, run_cmd


@click.command("cluster-info")
@click.pass_context
@click.option(
    "--upload/--no-upload",
    default=True,
    show_default=True,
    help="Upload the cluster dump TAR to a private Google Storage bucket shared with Indico",
)
@click.option(
    "--sudo/--no-sudo",
    default=False,
    show_default=False,
    help="Use sudo to run cluster dump (useful if running into permission issues)",
)
@options_wrapper()
def cluster_info(ctx, *, upload, sudo, deployment_root, **kwargs):
    """
    Load and package all the information on the running cluster
    into a local TAR file.
    """
    dumpdir = Path(deployment_root) / f"cluster_info_{time.strftime('%Y%m%d-%H%M%S')}"
    secho(f"Pulling cluster info...", fg="blue")
    sudo = "sudo " if sudo else ""
    run_cmd(
        f"""
        {sudo}kubectl cluster-info dump -o yaml --output-directory={dumpdir};
        {sudo}kubectl get secrets --namespace=default -o yaml > {dumpdir}/secrets.yaml;
        {sudo}kubectl get configmaps --namespace=default -o yaml > {dumpdir}/configmaps.yaml;
        {sudo}kubectl get ingress --namespace=default -o yaml > {dumpdir}/ingress.yaml;
        {sudo}kubectl get pv --namespace=default -o yaml > {dumpdir}/pv.yaml;
        {sudo}kubectl get pvc --namespace=default -o yaml > {dumpdir}/pvc.yaml;
        """,
        silent=True,
    )
    dump_tar = str(dumpdir.resolve()) + ".tar.gz"
    with tarfile.open(dump_tar, "w:gz") as tar:
        tar.add(dumpdir, arcname="")

    secho(f"Cluster information saved to {dump_tar}", fg="green")
    shutil.rmtree(dumpdir, ignore_errors=True)
    if not upload:
        return

    cluster_user = (
        current_user(clean=True) if auth_with_gsutil(deployment_root) else CLUSTER_NAME
    )
    if not cluster_user:
        secho(
            "Cannot authenticate with Google - please upload cluster info manually",
            fg="yellow",
        )
        return

    result = run_cmd(
        f"gsutil cp {dump_tar} gs://client_{cluster_user}/{Path(dump_tar).name} 2>&1",
        silent=True,
    )
    msgs = (
        (
            "Operaction completed",
            "Cluster information successfully shared with Indico",
            "green",
        ),
        (
            "BucketNotFoundException",
            f"Bucket for user {cluster_user} does not exist. Please ask Indico to create this and try sharing again, or manually share your cluster info TAR",
            "yellow",
        ),
        (
            "",
            f"Unable to share cluster info with Indico. Please share the cluster info TAR manually",
            "red",
        ),
    )
    for res, msg, color in msgs:
        if res in result:
            secho(msg, fg=color)
            break



from indico_install.utils import run_cmd


def check(conf):
    assert int(run_cmd("kubectl get nodes | wc -l")) >= 2, "At least 2 nodes are required in the cluster"


def create(conf):
    run_cmd("kubectl create -f https://raw.githubusercontent.com/NVIDIA/k8s-device-plugin/1.0.0-beta/nvidia-device-plugin.yml")



import datetime
import json
import time
from collections import OrderedDict
from copy import deepcopy

from click import prompt, secho

from indico_install.config import REMOTE_TEMPLATES_PATH, merge_dicts, yaml
from indico_install.utils import (
    current_user,
    diff_dicts,
    get_deployed_frontend,
    get_deployed_images,
    load_from_configmap,
    run_cmd,
    string_to_tag,
)

TRACKER_CONFIGMAP = "cluster-manager"
TRACKER_CONFIGMAP_BACKUP = "cluster-manager-backup"
LOCK_TIMEOUT = 60  # seconds to wait for configmap lock


"""
Updraft Library
"""


def get_services_yaml_from_tag(updraft_tag, apps_only=True):
    remote_services_yaml = REMOTE_TEMPLATES_PATH + updraft_tag + "/services.yaml"
    out = run_cmd(f"wget -qO- {remote_services_yaml}", silent=True)
    services = yaml.safe_load(out)
    services["services"] = {
        s: v
        for s, v in services["services"].items()
        if v.get("group", None) not in ("static", "configs") and "nginx" not in s
    }
    return services


def get_services_from_tag(updraft_tag):
    services = get_services_yaml_from_tag(updraft_tag, apps_only=True)
    results = {}
    for app in sorted(services["services"]):
        app_img = services["images"].get(app.replace("-", "")) or next(
            (
                services["images"][i]
                for i in services["images"].keys()
                if app.split("-")[0] in i
            ),
            None,
        )
        if app_img:
            results[app] = app_img
    results["stratosphere"] = services["frontend"]["hash"]
    return results


class ClusterManager(object):
    """
    TODO: make this hold state of the actual input file
    so we have documentation on what this looks like
    Probably with versions!
    """

    information_keys = ("indico_version", "cluster_config")

    def __init__(self, reconcile=False, input_yaml=None):
        """
        Initializes a version cluster_manager for the current cluster
        and preloads with existing data
        """
        self.indico_version = None
        self._lock = None
        self.cm_exists = False
        self.cluster_config = OrderedDict({})

        self.load_from_cluster()
        if not self.cm_exists:
            if not input_yaml:
                secho(
                    f"Input (cluster) yaml not provided to initialize Cluster Manager",
                    fg="yellow",
                )
                return
            with open(input_yaml, "r") as f:
                self.cluster_config = yaml.safe_load(f)
                self.indico_version = string_to_tag(prompt("Last known release tag"))
        elif input_yaml:
            secho(f"Ignoring input yaml. Manager exists", fg="yellow")

        if reconcile:
            self.reconcile()

    @property
    def current_state(self):
        state = {"stratosphere": self.cluster_config["frontend"].get("hash")}
        for svc, val in self.cluster_config["services"].items():
            img = val.get("values", {}).get("image")
            if not img or "nginx" in svc or "nginx" in img:
                continue
            state[svc] = img
        return state

    @property
    def registry(self):
        return self.cluster_config.get("dockerRegistry", "gcr.io/new-indico") + "/"

    @classmethod
    def diff(cls, version_1, version_2):
        old_service_hashes = get_services_from_tag(version_1)
        new_service_hashes = get_services_from_tag(version_2)
        return diff_dicts(old_service_hashes, new_service_hashes)

    def clean_services(self):
        updates = {}
        if "services" in self.cluster_config:
            services = deepcopy(self.cluster_config["services"])
            for svc, val in self.cluster_config["services"].items():
                if "image" not in val.get("values", {}):
                    continue
                del val["values"]["image"]
                val["values"].pop("updated_at", None)
                val["values"].pop("updated_by", None)
                # we don't want empty values floating around either
                if not val["values"]:
                    del val["values"]
            updates["services"] = services
        if "hash" in self.cluster_config["frontend"]:
            updates["frontend"] = {"hash": self.cluster_config["frontend"].pop("hash")}
        return updates

    def load_from_cluster(self, backup=False):
        """
        Use the version configmap on the cluster to populate this Tracker instance
        """
        conf = TRACKER_CONFIGMAP_BACKUP if backup else TRACKER_CONFIGMAP
        known_info = load_from_configmap(conf, data_parser=json.loads)
        if known_info:
            known_info = known_info["data"]
            self.cm_exists = True
            self.indico_version = known_info.get("indico_version")
            self.cluster_config = OrderedDict(
                known_info.get("cluster_config", self.cluster_config)
            )
        return known_info is not None

    def edit_cluster_config(self, changes=None, version=None):
        """
        Take a patch for cluster config and apply
        """
        if changes:
            self.cluster_config = merge_dicts(self.cluster_config, changes)
        if version:
            self.indico_version = version

    def _already_locked(self):
        """
        Determine if configmap is locked by someone other than us
        Return:
            - None if no lock
            - 0 if the lock is owned by us
            - True if a lock exists, is not owned by us, and is still valid
        """
        out = run_cmd(
            f"kubectl get cm {TRACKER_CONFIGMAP} --show-labels --no-headers 2>&1"
            "| awk '{print $4}'",
            silent=True,
        )
        if "NotFound" in out:
            return None
        lock_label = next((i for i in out.split(",") if i.startswith("lock")), None)
        if not lock_label:
            return

        lock = float(lock_label.split("=", 1)[1])
        if lock == self._lock:
            return 0
        return time.time() - lock < LOCK_TIMEOUT

    def lock(self, retries=15):
        """
        Locks configmap for LOCK_TIMEOUT seconds
        Raises TimeoutError if existing lock is not released during retries
        No op if manager does not exist
        """
        if not self.cm_exists:
            return
        curr_lock = self._already_locked()
        if curr_lock == 0:
            return self._lock

        needed_to_wait = curr_lock
        while curr_lock and retries:
            retries -= 1
            secho("Waiting for lock...", fg="yellow")
            time.sleep(2)
            curr_lock = self._already_locked()

        if curr_lock:
            raise TimeoutError("Lock was not acquired. Please try again later")

        self._lock = time.time()
        run_cmd(f"kubectl label cm {TRACKER_CONFIGMAP} lock={self._lock}", silent=True)
        if needed_to_wait:
            self.load_from_cluster()
        return self._lock

    def unlock(self):
        """
        Unlocks config manager only if the lock is owned by us
        """
        if not self.cm_exists:
            return
        curr_lock = self._already_locked()
        if curr_lock != 0:
            return
        run_cmd(f"kubectl label cm {TRACKER_CONFIGMAP} lock-", silent=True)

    def reconcile(self):
        """
        If there is a difference between what's on the cluster
        and the ClusterManager, update self
        """
        info = get_deployed_images()

        self.edit_cluster_config(
            changes={
                "frontend": {"hash": get_deployed_frontend()},
                "services": {
                    k: {"values": {"image": v.replace(self.registry, "")}}
                    for k, v in info.items()
                },
            }
        )

    def update_from_current(self, services):
        """
        Similar to update, but we pull the images of the services
        directly from the cluster. Unlike reconciling, we associate
        the changes with the current user and time
        """
        info = {k: v for k, v in get_deployed_images().items() if k in services}
        updated_fe = (
            get_deployed_frontend()
            if next((s for s in services if "app-edge-nginx" in s), None)
            else None
        )
        self.update(state=info, frontend=updated_fe)

    def update(self, state: dict = None, frontend: str = None, save=True):
        """
        Version is some release tag
        state is a dictionary of {svc: image}
        """
        if not state and not frontend:
            return
        changes = {}
        curr_user = current_user()
        curr_time = str(datetime.datetime.now())
        if state:
            changes["services"] = OrderedDict(
                {
                    k: {"values": {
                        "image": v.replace(self.registry, ""),
                        "updated_by": curr_user,
                        "updated_at": curr_time,
                    }}
                    for k, v in sorted(state.items())
                }
            )
        if frontend:
            changes["frontend"] = {
                "hash": frontend,
                "updated_by": curr_user,
                "updated_at": curr_time,
            }

        self.lock()
        try:
            self.edit_cluster_config(changes)

            if save:
                self.save()
        finally:
            self.unlock()

    def to_dict(self):
        return OrderedDict({k: getattr(self, k) for k in self.information_keys})

    def to_configmap(self, backup=False):
        return {
            "apiVersion": "v1",
            "kind": "ConfigMap",
            "metadata": {
                "name": TRACKER_CONFIGMAP_BACKUP if backup else TRACKER_CONFIGMAP,
            },
            "data": OrderedDict({k: json.dumps(v) for k, v in self.to_dict().items()}),
        }

    def to_str(self):
        return yaml.dump(self.to_dict(), default_flow_style=False)

    def save(self, backup=False):
        """
        Update configmap on cluster with self
        We use json instead of YAML to avoid issues with boolean encoding
        """
        run_cmd(
            f"echo '{json.dumps(self.to_configmap(backup=backup))}' | kubectl apply -f -",
            silent=True,
        )

    def diff_version(self, tag=None):
        tag = tag or self.indico_version
        if not tag or tag.startswith("GIT: "):
            return
        old_service_hashes = get_services_from_tag(tag)
        return diff_dicts(old_service_hashes, self.current_state)



from pathlib import Path

import click

from indico_install.kube.svc.restart import restart
from indico_install.setup import configure_gcr_key
from indico_install.utils import options_wrapper, run_cmd
from indico_install.cluster_manager import ClusterManager


def find_dependent_services(updated_services: list) -> set:
    """
    From a list of services, determine if any configs or secrets have been updated
    and return a list of deployment and statefulset names that need to be restarted
    to pick up changes. Determined if the services use any of the <updated_confs>
    in their envFrom or volumes.
    """
    svc_types_to_check = ("statefulset", "deployment", "configmap", "secret")
    updated_confs = [
        ups.split("/", 1)[1]
        for ups in updated_services
        if ups.startswith("secret/") or ups.startswith("configmap/")
    ]
    updated_services = {
        ups.split("/", 1)[1]: idx
        for idx, ups in enumerate(updated_services)
        if any(ups.startswith(svc_type) for svc_type in svc_types_to_check)
    }
    # We bank on the fact that names of deployments, statefulsets, configs, secrets are all unique

    dependency_info = []
    dependent_services = set([])
    # 1. Get all the conf dependencies for our resources
    for svc_type in ("deployment", "statefulset"):
        dependency_info.extend(
            run_cmd(
                f"kubectl get {svc_type} "
                "--no-headers -o custom-columns=NAME:.metadata.name,"
                "ENVFROM:{.spec.template.spec.containers[0].envFrom[*]},"
                "CONFIGMAPVOL:{.spec.template.spec.volumes[*]}",
                silent=True,
            ).splitlines()
        )

    # 2. For each conf, find the list of dependent resources
    for conf in updated_confs:
        conf_order = updated_services[conf]
        conf_deps = set([ds.split("  ", 1)[0] for ds in dependency_info if conf in ds])
        # remove any dependencies that were updated after the conf was updated
        conf_deps -= set(
            [cd for cd in conf_deps if updated_services.get(cd, 0) > conf_order]
        )
        dependent_services.update(conf_deps)

    return dependent_services


@click.command("apply")
@click.pass_context
@click.argument("service", required=False)
@click.option(
    "--restart-deps/--no-restart-deps",
    default=True,
    show_default=True,
    help="Restart services affected by updated configmap or secrets",
)
@options_wrapper()
def apply(ctx, restart_deps=True, service=None, deployment_root=None, yes=False, cluster_manager=None, **kwargs):
    """
    Apply templates in the "generated/" directory.
    Apply only the template files matching <SERVICE> if provided.
    Creates the image pull secret if missing.
    Performs a dry-run for confirmation before applying. Skip dry-run with --yes
    """
    generated_dir = Path(deployment_root) / "generated"
    if not generated_dir.is_dir():
        click.secho(f"'generated/' directory not found in {deployment_root}")
        return

    configure_gcr_key(deployment_root)

    # These are services that were actually modified
    updated_services = []

    if service is not None:
        templates = run_cmd(
            f"ls {generated_dir} | grep '{service}'", silent=True
        ).splitlines()
        for t in templates:
            results = run_cmd(
                f"kubectl apply -f {generated_dir / t} --dry-run 2>&1 | grep -v 'support dry-run'"
            )

        if yes or click.confirm("Ready to apply these changes?"):
            outs = [run_cmd(f"kubectl apply -f {generated_dir / t}") for t in templates]
            updated_services.extend(
                [
                    line.split(" ", 1)[0]
                    for out in outs
                    for line in out.splitlines()
                    if any(status in line for status in ("created", "configured"))
                ]
            )
    else:
        results = run_cmd(
            f"kubectl apply -R -f {generated_dir} --dry-run 2>&1 | grep -v 'support dry-run'",
            silent=True,
        )
        click.echo("\n".join(results.split(",")) + "\n")

        if yes or click.confirm("Ready to apply these changes?"):
            out = run_cmd(f"kubectl apply -R -f {generated_dir}", silent=True)
            for line in out.splitlines():
                color = "red"
                if "unchanged" in line:
                    color = "yellow"
                elif "configured" in line or "created" in line:
                    color = "green"
                    updated_services.append(line.split(" ", 1)[0])
                click.secho(line, fg=color)

    # Now we reroll any dependent services as necessary
    # And save information with the version cluster_manager
    if updated_services:
        cluster_manager = cluster_manager or ClusterManager()
        cluster_manager.update_from_current([s.split("/", 1)[-1] for s in updated_services])
        updated_confs = [
            ups
            for ups in updated_services
            if ups.startswith("secret/") or ups.startswith("configmap/")
        ]
        if not updated_confs:
            return
        if restart_deps:
            dependent_services = find_dependent_services(updated_services)
            if not dependent_services:
                click.secho(
                    f"No dependent services found that need to be restarted",
                    fg="yellow",
                )
                return
            for dep_service in dependent_services:
                ctx.invoke(restart, service=dep_service, contains=False, wait="2m")
        else:
            click.secho(f"{', '.join(updated_confs)} were updated. Make sure to restart dependent services")



from indico_install.infra.eks.config import session
from ipaddress import ip_network
from click import secho, prompt


def find_gw(info, possible_gws):
    # Todo: Use some ip magic
    gws = {g["GatewayName"]: g for g in possible_gws}
    curr_name = info.get("cluster", {}).get("gatewayName", None)
    if curr_name and curr_name in gws:
        return gws[curr_name]

    gw_options = "\n  ".join(list(gws.keys()))
    gw = None
    while not gw:
        gw = prompt(f"Possible gateways:\n  {gw_options}\nWhich gateway?")
        gw = gw if gw in gws else None
    return gws[gw]



@session
def gw_info(info, session=None):
    gws = session.SGWClient.list_gateways()["Gateways"]
    assert len(gws), "No gateways found"
    gw = gws[0] if len(gws) == 1 else find_gw(info, gws)
    gw_arn = gw["GatewayARN"]
    fss = session.SGWClient.list_file_shares(GatewayARN=gw_arn)["FileShareInfoList"]
    shares = session.SGWClient.describe_nfs_file_shares(FileShareARNList=[fs["FileShareARN"] for fs in fss])["NFSFileShareInfoList"]
    client_fs = next((share for share in shares if share["Path"] == info["clusterVolumes"]["rwx"]["nfs"]["path"]), None)
    indico_fs = next((share for share in shares if share["Path"] == info["clusterVolumes"]["rox"]["nfs"]["path"]), None)
    assert client_fs and indico_fs, "Missing a file share in your storage gateway"
    return {
        "gw_arn": gw_arn,
        "indico_fs": indico_fs,
        "client_fs": client_fs,
    }



def is_covered_by(cidr, bigger_cidr):
    covered = False
    # subnet_of is Python 3.7+, compare_networks was deprecated in 3.7
    try:
        covered = cidr.subnet_of(bigger_cidr)
    except AttributeError:
        covered = cidr.compare_networks(bigger_cidr) >= 0
    return covered

@session
def cluster_cidrs(info, session=None):
    cluster = session.EKSClient.describe_cluster(name=info["cluster"]["name"])[
        "cluster"
    ]
    try:
        return [
            ip_network(s["CidrBlock"])
            for s in session.EC2Client.describe_subnets(
                SubnetIds=cluster["resourcesVpcConfig"]["subnetIds"]
            )["Subnets"]
        ]
    except Exception as e:
        secho(f"Error getting CIDR blocks of cluster subnets: {e}", fg="orange")
    return []



import json
from collections import OrderedDict
import re

from indico_install.utils.base import run_cmd

APP_MAP = {
    "review": "moonbow",
    "teach": "crowdlabel",
    "auth": "noct",
    "datasets": "sharknado",
    "discover": "elmosfire",
    "default": "default",
    "kit": "indiKit",
}

STRATOSEARCH = re.compile(r"set \$clientversion ([^;]*);")


def get_nginx_conf():
    """ Get nginx conf from app-edge """
    return run_cmd(
        """kubectl get configmap app-edge-nginx-conf -o json | jq '.data["nginx.conf"]'""",
        silent=True,
    )


def get_app_hashes():
    output = run_cmd(
        r"""echo -e $(kubectl get configmap app-edge-nginx-conf -o json | jq '.data["nginx.conf"]') | grep -A 10 'map \$appname \$clientloc'""",
        silent=True,
    )
    output = output[output.find("{") + 1 : output.find("}")].strip()
    output = output.split()

    app_hashes = {}
    for idx in range(0, len(output), 2):
        app_hash = output[idx + 1]
        app_hashes[APP_MAP[output[idx]]] = app_hash[app_hash.rfind("/") + 1 : -1]
    return app_hashes


def get_non_matching_images(configs, only_first=False):
    output = run_cmd(
        "kubectl get deploy -o wide  | grep indico | awk '{print $1\"=\"$7s}'",
        silent=True,
    )
    images = {
        x[0]: ("deployment", x[1])
        for x in [y.split("=") for y in output.split("\n") if y.strip()]
    }

    output = run_cmd(
        "kubectl get statefulset -o wide  | grep indico | awk '{print $1\"=\"$5}'",
        silent=True,
    )

    images.update(
        {
            x[0]: ("statefulset", x[1])
            for x in [y.split("=") for y in output.split("\n") if y.strip()]
        }
    )
    for (deployment, (resource, image)) in images.items():
        if image.startswith("gcr.io/new-indico"):
            image = image[len("gcr.io/new-indico") + 1 :]
        elif image.startswith("indicoio"):
            image = image[len("indicoio") + 1 :]

        images[deployment] = (resource, image)

    for app, saved_image in configs["images"].items():
        for (deployment, (resource, cluster_image)) in images.items():
            formatted_deployment = deployment.replace("-", "").replace("_", "").lower()

            if app.lower() == formatted_deployment or app.lower() in cluster_image.replace(
                ".", ""
            ).replace(
                "-", ""
            ).replace(
                "_", ""
            ):
                if saved_image != cluster_image:
                    yield (app, resource, deployment, saved_image, cluster_image)
                if only_first:
                    break


def get_deployed_images(filter_nginx=True):
    out = run_cmd(
        "kubectl get deployments -l 'inditype in (service, celerytask)' "
        "-o custom-columns=NAME:.metadata.name,IMAGE:{.spec.template.spec.containers[0].image} --no-headers",
        silent=True,
    ).splitlines()
    out.extend(
        run_cmd(
            "kubectl get statefulsets -l 'inditype in (service, celerytask)' "
            "-o custom-columns=NAME:.metadata.name,IMAGE:{.spec.template.spec.containers[0].image} --no-headers",
            silent=True,
        ).splitlines()
    )
    return OrderedDict(
        {
            tuple(line.strip().split(None, 1))
            for line in sorted(out)
            if not (filter_nginx and "nginx" in line)
        }
    )


def get_deployed_frontend():
    conf = run_cmd("kubectl get configmap app-edge-nginx-conf -o json", silent=True)
    return STRATOSEARCH.search(conf).group(1)


def load_from_configmap(configmap_name, data_parser=None):
    out = run_cmd(f"kubectl get configmap {configmap_name} -o json 2>&1", silent=True)
    if "Error from server" in out:
        return
    try:
        res = json.loads(out)
        if data_parser:
            res["data"] = {k: data_parser(v) for k, v in res["data"].items()}
        return res
    except Exception:
        pass
    return



import os
from functools import wraps

import boto3
from cached_property import cached_property
from click import prompt

from indico_install.config import merge_dicts
from indico_install.infra.input_utils import postgres_input

access_key = os.getenv("AWS_ACCESS_KEY_ID")
access_secret = os.getenv("AWS_SECRET_ACCESS_KEY")
aws_region = os.getenv("AWS_REGION") or os.getenv("AWS_DEFAULT_REGION")
aws_profile = os.getenv("AWS_PROFILE") or os.getenv("AWS_DEFAULT_PROFILE", "default")


def ask_for_infra_input(conf):
    """
    Conf is a full dictionary
    Root is the key path we've followed to get to conf
    ask_all is whether or not to re-ask for input for existing values
    """
    data_gw_details = conf["clusterVolumes"]["rwx"]["nfs"]
    api_gw_details = conf["clusterVolumes"]["rox"]["nfs"]
    gateway_ip = prompt(
        f"What is your gateway IP?", type=str, default=data_gw_details.get("server")
    )

    client_bucket_name = prompt(
        f"What is the name of your read-write S3 bucket?",
        type=str,
        default=data_gw_details.get("path"),
    )
    if client_bucket_name and not client_bucket_name.startswith("/"):
        client_bucket_name = f"/{client_bucket_name}"

    model_bucket_name = prompt(
        f"What is the name of your read-only S3 bucket?",
        type=str,
        default=api_gw_details.get("path"),
    )
    if model_bucket_name and not model_bucket_name.startswith("/"):
        model_bucket_name = f"/{model_bucket_name}"

    conf["clusterVolumes"]["rwx"]["nfs"] = {
        "server": gateway_ip,
        "path": client_bucket_name,
    }
    conf["clusterVolumes"]["rox"]["nfs"] = {
        "server": gateway_ip,
        "path": model_bucket_name,
    }

    postgres_input(conf)

    cluster_name = prompt(
        "What is the name of your EKS cluster?",
        type=str,
        default=conf.get("cluster", {}).get("name"),
    )
    gateway_name = prompt(
        "What is the name of your storage gateway?",
        type=str,
        default=conf.get("cluster", {}).get("gatewayName"),
    )
    internal_elb_annotation = "service.beta.kubernetes.io/aws-load-balancer-internal"
    cluster_private = prompt(
        "Is your cluster private?",
        type=bool,
        default=internal_elb_annotation
        in conf.get("services", {}).get("app-edge", {}).get("values", {}).get("annotations", {}),
    )
    if cluster_private:
        conf["services"] = merge_dicts(
            conf.get("services", {}),
            {
                "app-edge": {
                    "values": {
                        "annotations": {
                            "service.beta.kubernetes.io/aws-load-balancer-internal": "0.0.0.0/0"
                        }
                    }
                }
            },
        )
    else:
        conf.get("services", {}).get("app-edge", {}).get("values", {}).get("annotations", {}).pop(internal_elb_annotation, None)

    conf["cluster"] = {"name": cluster_name, "gatewayName": gateway_name}
    return conf


def session(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        kwargs = kwargs or {}
        if not kwargs.get("session"):
            kwargs["session"] = Session()
        return f(*args, **kwargs)

    return wrapper


class Session(object):
    def __init__(self):
        if not ((access_key and access_secret) or aws_profile):
            self._boto_session = None
        else:
            self._boto_session = boto3.session.Session(
                region_name=aws_region,
                aws_access_key_id=access_key,
                aws_secret_access_key=access_secret,
                profile_name=aws_profile,
            )

    @cached_property
    def ASGClient(self):
        return self._boto_session.client("autoscaling")

    @cached_property
    def EC2Client(self):
        return self._boto_session.client("ec2")

    @cached_property
    def EC2Resource(self):
        return self._boto_session.resource("ec2")

    @cached_property
    def EKSClient(self):
        return self._boto_session.client("eks")

    @cached_property
    def IAMClient(self):
        return self._boto_session.client("iam")

    @cached_property
    def IAMResource(self):
        return self._boto_session.resource("iam")

    @cached_property
    def RDSClient(self):
        return self._boto_session.client("rds")

    @cached_property
    def S3Client(self):
        return self._boto_session.client("s3")

    @cached_property
    def SGWClient(self):
        return self._boto_session.client("storagegateway")



from pathlib import Path

import click
from indico_install.utils import base64file, convertb64, get_value, options_wrapper, decodeb64
from indico_install.cluster_manager import ClusterManager


@click.command("encode")
@click.pass_context
@click.option("--value", help="Encode this value")
@click.option(
    "--from-key",
    help="Encode value found at this key in your cluster config. Example: postgres.app.password",
)
@click.option("--from-file", help="Encode the contents of this file")
@click.option(
    "--save-key", help="Save the value to this key in cluster config. Example: tls.cert"
)
@click.option(
    "--stdout/--no-stdout",
    default=True,
    show_default=True,
    help="Print the encoded result to the screen",
)
@options_wrapper()
def encode(
    ctx,
    value=None,
    from_key=None,
    save_key=None,
    from_file=None,
    stdout=True,
    **kwargs,
):
    """
    Base64 encode a string from a provided source

    value overrides from-key, which overrides from-file
    """
    cluster_manager = None
    assert any(
        [value, from_key, from_file]
    ), "Must provide one of --value, --from-key, --from-file"
    if from_key or save_key:
        cluster_manager = ClusterManager()

    if value:
        encoded_value = convertb64(value)
    elif from_key:
        keys = from_key.split(".")
        value = get_value(cluster_manager.cluster_config, keys)
        assert isinstance(value, str), f"Value of {from_key} must be a string"
        encoded_value = convertb64(value)
    else:
        assert Path(from_file).is_file(), "--from-file could not be found"
        encoded_value = base64file(from_file)

    if stdout:
        click.echo(encoded_value)

    if save_key:
        cluster_manager.lock()
        keys = save_key.split(".")
        save_value = get_value(cluster_manager.cluster_config, keys[:-1])
        save_value[keys[-1]] = encoded_value
        cluster_manager.save()
        cluster_manager.unlock()

    return encoded_value


@click.command("decode")
@click.pass_context
@click.option("--value", help="Decode this value")
@click.option(
    "--from-key",
    help="Decode value found at this key in cluster config. Example: postgres.app.password",
)
@click.option("--save-file", help="Save the decoded value to this file")
@click.option(
    "--save-key", help="Save the value to this key in cluster config. Example: tls.cert"
)
@click.option(
    "--stdout/--no-stdout",
    default=True,
    show_default=True,
    help="Print the decoded result to the screen",
)
@options_wrapper()
def decode(
    ctx,
    value=None,
    from_key=None,
    save_key=None,
    save_file=None,
    stdout=True,
    **kwargs,
):
    """
    Decode a base64-encoded string from a provided source

    value overrides from-key
    """
    cluster_manager = None
    assert any([value, from_key]), "Must provide one of --value, --from-key"
    if from_key or save_key:
        cluster_manager = ClusterManager()

    if value:
        decoded_value = decodeb64(value)
    else:
        keys = from_key.split(".")
        value = get_value(cluster_manager.cluster_config, keys)
        assert isinstance(value, str), f"Value of {from_key} must be a string"
        decoded_value = decodeb64(value)

    if stdout:
        click.echo(decoded_value)

    if save_key:
        cluster_manager.lock()
        keys = save_key.split(".")
        save_value = get_value(cluster_manager.cluster_config, keys[:-1])
        save_value[keys[-1]] = decoded_value
        cluster_manager.save()
        cluster_manager.unlock()

    if save_file:
        with open(save_file, "w") as sfile:
            sfile.write(decoded_value)

    return decoded_value



import click
from indico_install.sync import push, pull
from indico_install.helm import apply, render, upload
from indico_install.generate import generate
from indico_install.infra import infra
from indico_install.kube import pod, svc
from indico_install.package import package
from indico_install.encoder import encode, decode
from indico_install.updraft import updraft

command_groups = [
    pull,
    push,
    apply,
    render,
    generate,
    infra,
    package,
    pod,
    svc,
    encode,
    decode,
    updraft,
    upload,
]


@click.group("indico_cli")
@click.version_option()
@click.pass_context
def indico_cli(ctx):
    r"""
    Indico's CLI tool for cluster deployment and management.

    See `indico COMMAND [SUBCOMMAND, ] --help` for more information

    Scripts:

      These are utility scripts encapsulating the most common commands used in cluster management. They come installed with the indico CLI tool.

        kd [PODREGEX='']: describe all pods matching PODREGEX (wraps `kubectl describe pods`). Example: `kd custom-predict`

        kp [PODREGEX='']: list pods matching PODREGEX (wraps `kubectl get pods -o wide`). Example: `kp` or `kp custom`

        klogs [PODREGEX='']: tail logs for pods matching PODREGEX. Example: `klogs custom` or `klogs custom\|moonbow`

        kube exec PODREGEX [COMMAND='bash']: run COMMAND as a TTY in the first pod matching PODREGEX (wraps `kubectl exec -it <pod>`). Example: `kube exec fog`

        kube scale SERVICEREGEX AMOUNT: scale the number of pods for services matching SERVICEREGEX to AMOUNT.
        Will save settings in your input yaml. Alias of `indico svc scale`

        kube update SERVICEREGEX: roll the pods of all services matching SERVICEREGEX. Alias of `indico svc restart`

    """
    ctx.ensure_object(dict)


for command_group in command_groups:
    indico_cli.add_command(command_group)



#!/usr/bin/env python3
import os
from subprocess import check_call

from setuptools import find_packages, setup

setup(
    name="indico_install",
    version="1.0.1rc0",
    description="An indico install platform",
    author="Indico",
    author_email="engineering@indico.io",
    packages=find_packages(),
    install_requires=[
        "click",
        "pyyaml==5.1.2",
        "requests",
        "psycopg2-binary",
        "cached_property",
        "setuptools",
        "google-cloud-storage==1.25.0",
    ],
    extras_require={"aks": ["azure-cli"], "eks": ["boto3==1.9.176"]},
    include_package_data=True,
    entry_points="""
        [console_scripts]
        indico=indico_install:indico_cli
    """,
    scripts=["bin/kube", "bin/cluster_info.sh"]
    + [f"bin/kube_bin/{filename}" for filename in os.listdir("bin/kube_bin")],
)

check_call(
    [
        "bash",
        "-c",
        'azcomm=$(command -v az); if ! [ -z "$azcomm" ]; then sed -i "s/^python /python3 /g" $azcomm; fi',
    ]
)
